@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -165,7 +165,7 @@ static int sql_start_result_metadata(void *ctx, uint num_cols, uint,
   DBUG_TRACE;
   DBUG_PRINT("info", ("resultcs->number: %d", resultcs->number));
   DBUG_PRINT("info", ("resultcs->csname: %s", resultcs->csname));
-  DBUG_PRINT("info", ("resultcs->name: %s", resultcs->name));
+  DBUG_PRINT("info", ("resultcs->m_coll_name: %s", resultcs->m_coll_name));
   pctx->num_cols = num_cols;
   pctx->resultcs = resultcs;
   pctx->current_col = 0;
@@ -554,7 +554,7 @@ static void dump_cs_info(const CHARSET_INFO *cs) {
 
   WRITE_VAL("\t\t[meta][charset result] number: %d\n", cs->number);
   WRITE_VAL("\t\t[meta][charset result] name: %s\n", cs->csname);
-  WRITE_VAL("\t\t[meta][charset result] collation: %s\n", cs->name);
+  WRITE_VAL("\t\t[meta][charset result] collation: %s\n", cs->m_coll_name);
   WRITE_VAL("\t\t[meta][charset result] sort order: %s\n", cs->sort_order);
 }
 
@@ -680,7 +680,7 @@ static void set_query_in_com_data(const char *query, union COM_DATA *cmd) {
 
 static void run_statement(MYSQL_SESSION session, const char *query,
                           struct st_plugin_ctx *ctx, bool generates_result_set,
-                          void *p MY_ATTRIBUTE((unused))) {
+                          void *p [[maybe_unused]]) {
   char buffer[STRING_BUFFER_SIZE];
   COM_DATA cmd;
 
@@ -693,7 +693,7 @@ static void run_statement(MYSQL_SESSION session, const char *query,
 again:
   ctx->reset();
   int fail = command_service_run_command(session, COM_QUERY, &cmd,
-                                         &my_charset_utf8_general_ci,
+                                         &my_charset_utf8mb3_general_ci,
                                          &protocol_callbacks, txt_or_bin, ctx);
   if (fail) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "run_statement code: %d\n",
@@ -718,7 +718,7 @@ again:
 
 void static change_current_db(MYSQL_SESSION session, const char *db,
                               struct st_plugin_ctx *ctx,
-                              void *p MY_ATTRIBUTE((unused))) {
+                              void *p [[maybe_unused]]) {
   char buffer[STRING_BUFFER_SIZE];
   COM_DATA cmd;
   cmd.com_init_db.db_name = db;
@@ -728,7 +728,7 @@ void static change_current_db(MYSQL_SESSION session, const char *db,
   WRITE_VAL("EXECUTING:[COM_INIT_DB][%s]\n", db);
   ctx->reset();
   int fail = command_service_run_command(
-      session, COM_INIT_DB, &cmd, &my_charset_utf8_general_ci,
+      session, COM_INIT_DB, &cmd, &my_charset_utf8mb3_general_ci,
       &protocol_callbacks, CS_TEXT_REPRESENTATION, ctx);
   if (fail)
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "change db code: %d\n", fail);
@@ -855,7 +855,7 @@ static int test_sql_service_plugin_init(void *p) {
   return 0;
 }
 
-static int test_sql_service_plugin_deinit(void *p MY_ATTRIBUTE((unused))) {
+static int test_sql_service_plugin_deinit(void *p [[maybe_unused]]) {
   DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Uninstallation.");
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
