@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,18 +22,22 @@
 
 #include "sql/parse_tree_hints.h"
 
-#include <stddef.h>
-#include <string.h>
+#include <assert.h>
+#include <cstddef>
+#include <cstdio>
+#include <cstring>
 
+#include "lex_string.h"
 #include "m_ctype.h"
 #include "m_string.h"
-#include "my_dbug.h"
+#include "my_alloc.h"
+
 #include "my_sqlcommand.h"
-#include "mysql/components/services/log_shared.h"
 #include "mysqld_error.h"
 #include "sql/derror.h"
 #include "sql/item_subselect.h"
 #include "sql/mysqld.h"  // table_alias_charset
+#include "sql/parse_tree_helpers.h"
 #include "sql/query_options.h"
 #include "sql/resourcegroups/resource_group_basic_types.h"
 #include "sql/resourcegroups/resource_group_mgr.h"
@@ -70,23 +74,24 @@ static Opt_hints_global *get_global_hints(Parse_context *pc) {
   for query block given by parse context,
   create Opt_hints_qb object if not exist.
 
-  @param pc   pointer to Parse_context object
+  @param pc      pointer to Parse_context object
+  @param select  pointer to Query_block object
 
   @return  pointer to Opt_hints_qb object,
            NULL if failed to create the object
 */
 
-static Opt_hints_qb *get_qb_hints(Parse_context *pc) {
-  if (pc->select->opt_hints_qb) return pc->select->opt_hints_qb;
+static Opt_hints_qb *get_qb_hints(Parse_context *pc, Query_block *select) {
+  if (select->opt_hints_qb) return select->opt_hints_qb;
 
   Opt_hints_global *global_hints = get_global_hints(pc);
-  if (global_hints == NULL) return NULL;
+  if (global_hints == nullptr) return nullptr;
 
   Opt_hints_qb *qb = new (pc->thd->mem_root)
-      Opt_hints_qb(global_hints, pc->thd->mem_root, pc->select->select_number);
+      Opt_hints_qb(global_hints, pc->thd->mem_root, select->select_number);
   if (qb) {
     global_hints->register_child(qb);
-    pc->select->opt_hints_qb = qb;
+    select->opt_hints_qb = qb;
     qb->set_resolved();
   }
   return qb;
@@ -113,11 +118,28 @@ static Opt_hints_qb *find_qb_hints(Parse_context *pc,
       static_cast<Opt_hints_qb *>(pc->thd->lex->opt_hints_global->find_by_name(
           qb_name, system_charset_info));
 
-  if (qb == NULL) {
-    hint->print_warn(pc->thd, ER_WARN_UNKNOWN_QB_NAME, qb_name, NULL, NULL,
-                     NULL);
+  //  Find query block using system name. Used for proper parsing of view body.
+  if (!qb) {
+    LEX *lex = pc->thd->lex;
+    for (Query_block *select = lex->all_query_blocks_list; select != nullptr;
+         select = select->next_select_in_list()) {
+      LEX_CSTRING sys_name;  // System QB name
+      char buff[32];         // Buffer to hold sys name
+      sys_name.str = buff;
+      sys_name.length = snprintf(buff, sizeof(buff), "%s%x", "select#",
+                                 select->select_number);
+
+      if (!cmp_lex_string(sys_name, *qb_name, system_charset_info)) {
+        qb = get_qb_hints(pc, select);
+        break;
+      }
+    }
   }
 
+  if (qb == nullptr)
+    hint->print_warn(pc->thd, ER_WARN_UNKNOWN_QB_NAME, qb_name, nullptr,
+                     nullptr, nullptr);
+
   return qb;
 }
 
@@ -203,7 +225,7 @@ bool PT_qb_level_hint::contextualize(Parse_context *pc) {
   if (super::contextualize(pc)) return true;
 
   Opt_hints_qb *qb = find_qb_hints(pc, &qb_name, this);
-  if (qb == NULL) return false;  // TODO: Should this generate a warning?
+  if (qb == nullptr) return false;  // TODO: Should this generate a warning?
 
   bool no_warn = false;   // If true, do not print a warning
   bool conflict = false;  // true if this hint conflicts with a previous hint
@@ -249,13 +271,14 @@ bool PT_qb_level_hint::contextualize(Parse_context *pc) {
         pc->select->add_base_options(SELECT_STRAIGHT_JOIN);
       break;
     default:
-      DBUG_ASSERT(0);
+      assert(0);
   }
 
   if (conflict ||
       // Set hint or detect if hint has been set before
       (qb->set_switch(switch_on(), type(), false) && !no_warn))
-    print_warn(pc->thd, ER_WARN_CONFLICTING_HINT, &qb_name, NULL, NULL, this);
+    print_warn(pc->thd, ER_WARN_CONFLICTING_HINT, &qb_name, nullptr, nullptr,
+               this);
 
   return false;
 }
@@ -283,15 +306,15 @@ void PT_qb_level_hint::append_args(const THD *thd, String *str) const {
       break;
     }
     case SUBQUERY_HINT_ENUM:
-      switch (args) {
-        case Item_exists_subselect::EXEC_MATERIALIZATION:
+      switch (static_cast<Subquery_strategy>(args)) {
+        case Subquery_strategy::SUBQ_MATERIALIZATION:
           str->append(STRING_WITH_LEN(" MATERIALIZATION"));
           break;
-        case Item_exists_subselect::EXEC_EXISTS:
+        case Subquery_strategy::SUBQ_EXISTS:
           str->append(STRING_WITH_LEN(" INTOEXISTS"));
           break;
         default:  // Exactly one of above strategies should always be specified
-          DBUG_ASSERT(false);
+          assert(false);
       }
       break;
     case JOIN_PREFIX_HINT_ENUM:
@@ -307,17 +330,22 @@ void PT_qb_level_hint::append_args(const THD *thd, String *str) const {
     case JOIN_FIXED_ORDER_HINT_ENUM:
       break;
     default:
-      DBUG_ASSERT(false);
+      assert(false);
   }
 }
 
 bool PT_hint_list::contextualize(Parse_context *pc) {
   if (super::contextualize(pc)) return true;
 
-  if (!get_qb_hints(pc)) return true;
+  if (!get_qb_hints(pc, pc->select)) return true;
 
   for (PT_hint **h = hints.begin(), **end = hints.end(); h < end; h++) {
-    if (*h != NULL && (*h)->contextualize(pc)) return true;
+    if (*h != nullptr) {
+      if (pc->thd->lex->sql_command == SQLCOM_CREATE_VIEW &&
+          !(*h)->supports_view())
+        continue;
+      if ((*h)->contextualize(pc)) return true;
+    }
   }
   return false;
 }
@@ -328,10 +356,11 @@ bool PT_table_level_hint::contextualize(Parse_context *pc) {
   if (table_list.empty())  // Query block level hint
   {
     Opt_hints_qb *qb = find_qb_hints(pc, &qb_name, this);
-    if (qb == NULL) return false;
+    if (qb == nullptr) return false;
 
     if (qb->set_switch(switch_on(), type(), false))
-      print_warn(pc->thd, ER_WARN_CONFLICTING_HINT, &qb_name, NULL, NULL, this);
+      print_warn(pc->thd, ER_WARN_CONFLICTING_HINT, &qb_name, nullptr, nullptr,
+                 this);
     return false;
   }
 
@@ -346,21 +375,22 @@ bool PT_table_level_hint::contextualize(Parse_context *pc) {
         qb_name.length > 0 ? &qb_name : &table_name->opt_query_block;
 
     Opt_hints_qb *qb = find_qb_hints(pc, qb_name_str, this);
-    if (qb == NULL) return false;
+    if (qb == nullptr) return false;
 
     Opt_hints_table *tab = get_table_hints(pc, table_name, qb);
     if (!tab) return true;
 
     if (tab->set_switch(switch_on(), type(), true))
       print_warn(pc->thd, ER_WARN_CONFLICTING_HINT,
-                 &table_name->opt_query_block, &table_name->table, NULL, this);
+                 &table_name->opt_query_block, &table_name->table, nullptr,
+                 this);
   }
 
   return false;
 }
 
 void PT_key_level_hint::append_args(const THD *thd, String *str) const {
-  if (type() == INDEX_MERGE_HINT_ENUM || type() == SKIP_SCAN_HINT_ENUM) {
+  if (is_compound_hint(type())) {
     for (uint i = 0; i < key_list.size(); i++) {
       const LEX_CSTRING *key_name = &key_list.at(i);
       str->append(STRING_WITH_LEN(" "));
@@ -376,7 +406,7 @@ bool PT_key_level_hint::contextualize(Parse_context *pc) {
   if (super::contextualize(pc)) return true;
 
   Opt_hints_qb *qb = find_qb_hints(pc, &table_name.opt_query_block, this);
-  if (qb == NULL) return false;
+  if (qb == nullptr) return false;
 
   Opt_hints_table *tab = get_table_hints(pc, &table_name, qb);
   if (!tab) return true;
@@ -384,19 +414,23 @@ bool PT_key_level_hint::contextualize(Parse_context *pc) {
   bool is_conflicting = false;
   if (key_list.empty())  // Table level hint
   {
-    if (tab->set_switch(switch_on(), type(), false)) {
+    if ((is_compound_hint(type()) &&
+         tab->get_compound_key_hint(type())->is_hint_conflicting(tab,
+                                                                 nullptr)) ||
+        tab->set_switch(switch_on(), type(), false)) {
       print_warn(pc->thd, ER_WARN_CONFLICTING_HINT, &table_name.opt_query_block,
-                 &table_name.table, NULL, this);
+                 &table_name.table, nullptr, this);
       return false;
     }
   }
 
   if (type() == INDEX_MERGE_HINT_ENUM && key_list.size() == 1 && switch_on()) {
     print_warn(pc->thd, ER_WARN_INVALID_HINT, &table_name.opt_query_block,
-               &table_name.table, NULL, this);
+               &table_name.table, nullptr, this);
     return false;
   }
 
+  Mem_root_array<Opt_hints_key *> key_hints(pc->thd->mem_root);
   for (size_t i = 0; i < key_list.size(); i++) {
     LEX_CSTRING *key_name = &key_list.at(i);
     Opt_hints_key *key =
@@ -408,22 +442,33 @@ bool PT_key_level_hint::contextualize(Parse_context *pc) {
       tab->register_child(key);
     }
 
-    if (key->set_switch(switch_on(), type(), true)) {
-      is_conflicting = true;
-      if (tab->is_compound_key_hint(type())) {
-        print_warn(pc->thd, ER_WARN_CONFLICTING_HINT,
-                   &table_name.opt_query_block, &table_name.table, NULL, this);
-        break;
-      } else
-        print_warn(pc->thd, ER_WARN_CONFLICTING_HINT,
-                   &table_name.opt_query_block, &table_name.table, key_name,
-                   this);
+    bool is_specified = tab->is_specified(type()) || key->is_specified(type());
+    if (is_specified && !is_compound_hint(type())) {
+      print_warn(pc->thd, ER_WARN_CONFLICTING_HINT, &table_name.opt_query_block,
+                 &table_name.table, key_name, this);
+      continue;
     }
+
+    if (is_specified ||
+        (is_compound_hint(type()) &&
+         tab->get_compound_key_hint(type())->is_hint_conflicting(tab, key))) {
+      is_conflicting = true;
+      print_warn(pc->thd, ER_WARN_CONFLICTING_HINT, &table_name.opt_query_block,
+                 &table_name.table, nullptr, this);
+      break;
+    }
+    key_hints.push_back(key);
   }
 
-  if (tab->is_compound_key_hint(type()) && !is_conflicting) {
-    tab->get_compound_key_hint(type())->set_pt_hint(this);
-    (void)tab->set_switch(switch_on(), type(), false);
+  if (!is_conflicting) {
+    for (size_t i = 0; i < key_hints.size(); i++) {
+      Opt_hints_key *key = key_hints.at(i);
+      key->set_switch(switch_on(), type(), true);
+    }
+    if (is_compound_hint(type())) {
+      tab->get_compound_key_hint(type())->set_pt_hint(this);
+      (void)tab->set_switch(switch_on(), type(), false);
+    }
   }
 
   return false;
@@ -434,12 +479,13 @@ bool PT_hint_qb_name::contextualize(Parse_context *pc) {
 
   Opt_hints_qb *qb = pc->select->opt_hints_qb;
 
-  DBUG_ASSERT(qb);
+  assert(qb);
 
   if (qb->get_name() ||                         // QB name is already set
       qb->get_parent()->find_by_name(&qb_name,  // Name is already used
                                      system_charset_info)) {
-    print_warn(pc->thd, ER_WARN_CONFLICTING_HINT, NULL, NULL, NULL, this);
+    print_warn(pc->thd, ER_WARN_CONFLICTING_HINT, nullptr, nullptr, nullptr,
+               this);
     return false;
   }
 
@@ -452,7 +498,7 @@ bool PT_hint_max_execution_time::contextualize(Parse_context *pc) {
 
   if (pc->thd->lex->sql_command != SQLCOM_SELECT ||  // not a SELECT statement
       pc->thd->lex->sphead ||                        // or in a SP/trigger/event
-      pc->select != pc->thd->lex->select_lex)        // or in a subquery
+      pc->select != pc->thd->lex->query_block)       // or in a subquery
   {
     push_warning(pc->thd, Sql_condition::SL_WARNING,
                  ER_WARN_UNSUPPORTED_MAX_EXECUTION_TIME,
@@ -463,7 +509,8 @@ bool PT_hint_max_execution_time::contextualize(Parse_context *pc) {
   Opt_hints_global *global_hint = get_global_hints(pc);
   if (global_hint->is_specified(type())) {
     // Hint duplication: /*+ MAX_EXECUTION_TIME ... MAX_EXECUTION_TIME */
-    print_warn(pc->thd, ER_WARN_CONFLICTING_HINT, NULL, NULL, NULL, this);
+    print_warn(pc->thd, ER_WARN_CONFLICTING_HINT, nullptr, nullptr, nullptr,
+               this);
     return false;
   }
 
@@ -479,12 +526,13 @@ bool PT_hint_sys_var::contextualize(Parse_context *pc) {
     return false;
   }
 
-  sys_var *sys_var = find_sys_var_ex(pc->thd, sys_var_name.str,
-                                     sys_var_name.length, true, false);
-  if (!sys_var) {
+  System_variable_tracker var_tracker =
+      System_variable_tracker::make_tracker(to_string_view(sys_var_name));
+  if (var_tracker.access_system_variable(pc->thd, {},
+                                         Suppress_not_found_error::YES)) {
     String str;
     str.append(STRING_WITH_LEN("'"));
-    str.append(sys_var_name.str, sys_var_name.length);
+    str.append(sys_var_name);
     str.append(STRING_WITH_LEN("'"));
     push_warning_printf(
         pc->thd, Sql_condition::SL_WARNING, ER_UNRESOLVED_HINT_NAME,
@@ -492,7 +540,7 @@ bool PT_hint_sys_var::contextualize(Parse_context *pc) {
     return false;
   }
 
-  if (!sys_var->is_hint_updateable()) {
+  if (!var_tracker.is_hint_updateable()) {
     String str;
     str.append(STRING_WITH_LEN("'"));
     str.append(sys_var_name.str, sys_var_name.length);
@@ -510,7 +558,8 @@ bool PT_hint_sys_var::contextualize(Parse_context *pc) {
         new (pc->thd->mem_root) Sys_var_hint(pc->thd->mem_root);
   if (!global_hint->sys_var_hint) return true;
 
-  return global_hint->sys_var_hint->add_var(pc->thd, sys_var, sys_var_value);
+  return global_hint->sys_var_hint->add_var(pc->thd, var_tracker,
+                                            sys_var_value);
 }
 
 bool PT_hint_resource_group::contextualize(Parse_context *pc) {
@@ -522,7 +571,7 @@ bool PT_hint_resource_group::contextualize(Parse_context *pc) {
     return false;
   }
 
-  if (pc->thd->lex->sphead || pc->select != pc->thd->lex->select_lex) {
+  if (pc->thd->lex->sphead || pc->select != pc->thd->lex->query_block) {
     pc->thd->resource_group_ctx()->m_warn =
         WARN_RESOURCE_GROUP_UNSUPPORTED_HINT;
     return false;
