@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,12 +22,16 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+#include "util/require.h"
 #include <ndb_global.h>
+#include "ndb_config.h"
 
 #include <TransporterRegistry.hpp>
 #include "Configuration.hpp"
 #include <ErrorHandlingMacros.hpp>
 #include "GlobalData.hpp"
+#include "portlib/NdbTCP.h"
+#include "transporter/TransporterCallback.hpp"
 
 #include <ConfigRetriever.hpp>
 #include <IPCConfig.hpp>
@@ -52,7 +56,6 @@
 
 #define JAM_FILE_ID 301
 
-extern EventLogger * g_eventLogger;
 
 extern Uint32 g_start_type;
 
@@ -94,8 +97,6 @@ Configuration::Configuration()
   _backupPath = 0;
   _initialStart = false;
   m_config_retriever= 0;
-  m_clusterConfig= 0;
-  m_clusterConfigIter= 0;
   m_logLevel= 0;
 }
 
@@ -114,6 +115,7 @@ Configuration::~Configuration(){
   if(m_logLevel) {
     delete m_logLevel;
   }
+  ndb_mgm_destroy_iterator(m_clusterConfigIter);
 }
 
 void
@@ -167,8 +169,6 @@ Configuration::fetch_configuration(const char* _connect_string,
     ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG, "Could not connect to ndb_mgmd", s);
   }
 
-  ConfigRetriever &cr= *m_config_retriever;
-
   if (allocated_nodeid)
   {
     // The angel has already allocated the nodeid, no need to
@@ -180,7 +180,8 @@ Configuration::fetch_configuration(const char* _connect_string,
 
     const int alloc_retries = 10;
     const int alloc_delay = 3;
-    globalData.ownId = cr.allocNodeId(alloc_retries, alloc_delay);
+    globalData.ownId =
+        m_config_retriever->allocNodeId(alloc_retries, alloc_delay);
     if(globalData.ownId == 0)
     {
       ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG,
@@ -190,9 +191,9 @@ Configuration::fetch_configuration(const char* _connect_string,
   }
   assert(globalData.ownId);
 
-  ndb_mgm_configuration * p = cr.getConfig(globalData.ownId);
-  if(p == 0){
-    const char * s = cr.getErrorString();
+  m_clusterConfig = m_config_retriever->getConfig(globalData.ownId);
+  if(!m_clusterConfig){
+    const char * s = m_config_retriever->getErrorString();
     if(s == 0)
       s = "No error given!";
     
@@ -203,26 +204,23 @@ Configuration::fetch_configuration(const char* _connect_string,
     ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG, "Could not fetch configuration"
 	      "/invalid configuration", s);
   }
-  if(m_clusterConfig)
-    free(m_clusterConfig);
-  
-  m_clusterConfig = p;
 
-  const ConfigValues * cfg = (ConfigValues*)m_clusterConfig;
-  cfg->pack_v1(m_clusterConfigPacked_v1);
+  const ConfigValues& cfg = m_clusterConfig.get()->m_config_values;
+  cfg.pack_v1(m_clusterConfigPacked_v1);
   if (OUR_V2_VERSION)
   {
-    cfg->pack_v2(m_clusterConfigPacked_v2);
+    cfg.pack_v2(m_clusterConfigPacked_v2);
   }
 
   {
     Uint32 generation;
-    ndb_mgm_configuration_iterator sys_iter(*p, CFG_SECTION_SYSTEM);
-    char buf[512];
+    ndb_mgm_configuration_iterator sys_iter(m_clusterConfig.get(),
+                                            CFG_SECTION_SYSTEM);
+    char sockaddr_buf[512];
     char* sockaddr_string =
-        Ndb_combine_address_port(buf, sizeof(buf),
-        m_config_retriever->get_mgmd_host(),
-        m_config_retriever->get_mgmd_port());
+        Ndb_combine_address_port(sockaddr_buf, sizeof(sockaddr_buf),
+                                 m_config_retriever->get_mgmd_host(),
+                                 m_config_retriever->get_mgmd_port());
 
     if (sys_iter.get(CFG_SYS_CONFIG_GENERATION, &generation))
     {
@@ -236,11 +234,12 @@ Configuration::fetch_configuration(const char* _connect_string,
     }
   }
 
-  ndb_mgm_configuration_iterator iter(* p, CFG_SECTION_NODE);
-  if (iter.find(CFG_NODE_ID, globalData.ownId)){
-    ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG, "Invalid configuration fetched", "DB missing");
+  ndb_mgm_configuration_iterator iter(m_clusterConfig.get(), CFG_SECTION_NODE);
+  if (iter.find(CFG_NODE_ID, globalData.ownId)) {
+    ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG, "Invalid configuration fetched",
+              "DB missing");
   }
-  
+
   if(iter.get(CFG_DB_STOP_ON_ERROR, &_stopOnError)){
     ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG, "Invalid configuration fetched", 
 	      "StopOnError missing");
@@ -297,8 +296,6 @@ Configuration::setupConfiguration(){
 
   DBUG_ENTER("Configuration::setupConfiguration");
 
-  ndb_mgm_configuration * p = m_clusterConfig;
-
   /**
    * Configure transporters
    */
@@ -310,7 +307,7 @@ Configuration::setupConfiguration(){
   }
 
   if (!IPCConfig::configureTransporters(globalData.ownId,
-                                        * p,
+                                        m_clusterConfig.get(),
                                         globalTransporterRegistry))
   {
     ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG,
@@ -319,9 +316,9 @@ Configuration::setupConfiguration(){
   }
 
   /**
-   * Setup cluster configuration data
+   * Setup cluster configuration for this node
    */
-  ndb_mgm_configuration_iterator iter(* p, CFG_SECTION_NODE);
+  ndb_mgm_configuration_iterator iter(m_clusterConfig.get(), CFG_SECTION_NODE);
   if (iter.find(CFG_NODE_ID, globalData.ownId)){
     ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG, "Invalid configuration fetched", "DB missing");
   }
@@ -397,7 +394,7 @@ Configuration::setupConfiguration(){
   iter.get(CFG_MIXOLOGY_LEVEL, &_mixologyLevel);
   if (_mixologyLevel)
   {
-    ndbout_c("Mixology level set to 0x%x", _mixologyLevel);
+    g_eventLogger->info("Mixology level set to 0x%x", _mixologyLevel);
     globalTransporterRegistry.setMixologyLevel(_mixologyLevel);
   }
 #endif
@@ -471,11 +468,8 @@ Configuration::setupConfiguration(){
   if (auto_thread_config == 0 &&
       thrconfigstring != nullptr && thrconfigstring[0] != 0)
   {
-    int res = m_thr_config.do_parse(thrconfigstring,
-                                    _realtimeScheduler,
-                                    _schedulerSpinTimer,
-                                    globalData.ndbRRGroups,
-                                    false);
+    int res = m_thr_config.do_parse(
+        thrconfigstring, _realtimeScheduler, _schedulerSpinTimer);
     if (res != 0)
     {
       ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG,
@@ -513,9 +507,7 @@ Configuration::setupConfiguration(){
                                       lqhthreads,
                                       classic,
                                       _realtimeScheduler,
-                                      _schedulerSpinTimer,
-                                      globalData.ndbRRGroups,
-                                      false);
+                                      _schedulerSpinTimer);
       if (res != 0)
       {
         ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG,
@@ -528,34 +520,30 @@ Configuration::setupConfiguration(){
   {
     if (thrconfigstring)
     {
-      ndbout_c("ThreadConfig: input: %s LockExecuteThreadToCPU: %s =>"
-               " parsed: %s",
-               thrconfigstring,
-               lockmask ? lockmask : "",
-               m_thr_config.getConfigString());
+      g_eventLogger->info(
+          "ThreadConfig: input: %s LockExecuteThreadToCPU: %s => parsed: %s",
+          thrconfigstring, lockmask ? lockmask : "",
+          m_thr_config.getConfigString());
     }
     else if (mtthreads == 0)
     {
-      ndbout_c("Automatic Thread Config: LockExecuteThreadToCPU: %s =>"
-               " parsed: %s",
-               lockmask ? lockmask : "",
-               m_thr_config.getConfigString());
+      g_eventLogger->info(
+          "Automatic Thread Config: LockExecuteThreadToCPU: %s => parsed: %s",
+          lockmask ? lockmask : "", m_thr_config.getConfigString());
     }
     else
     {
-      ndbout_c("ThreadConfig (old ndb_mgmd) LockExecuteThreadToCPU: %s =>"
-               " parsed: %s",
-               lockmask ? lockmask : "",
-               m_thr_config.getConfigString());
+      g_eventLogger->info(
+          "ThreadConfig (old ndb_mgmd) LockExecuteThreadToCPU: %s => parsed: %s",
+          lockmask ? lockmask : "", m_thr_config.getConfigString());
     }
   }
 
   ConfigValues* cf = ConfigValuesFactory::extractCurrentSection(iter.m_config);
 
-  if(m_clusterConfigIter)
-    ndb_mgm_destroy_iterator(m_clusterConfigIter);
-  m_clusterConfigIter = ndb_mgm_create_configuration_iterator
-    (p, CFG_SECTION_NODE);
+  ndb_mgm_destroy_iterator(m_clusterConfigIter);
+  m_clusterConfigIter = ndb_mgm_create_configuration_iterator(
+      m_clusterConfig.get(), CFG_SECTION_NODE);
 
   /**
    * This is parts of get_multithreaded_config
@@ -769,10 +757,10 @@ Configuration::getOwnConfigIterator() const {
   return m_ownConfigIterator;
 }
 
-const class ConfigValues*
+const ConfigValues*
 Configuration::get_own_config_values()
 {
-  return &m_ownConfig->m_config;
+  return &m_ownConfig->m_config_values;
 }
 
 
@@ -784,7 +772,7 @@ Configuration::getClusterConfigIterator() const {
 Uint32 
 Configuration::get_config_generation() const {
   Uint32 generation = ~0;
-  ndb_mgm_configuration_iterator sys_iter(*m_clusterConfig,
+  ndb_mgm_configuration_iterator sys_iter(m_clusterConfig.get(),
                                           CFG_SECTION_SYSTEM);
   sys_iter.get(CFG_SYS_CONFIG_GENERATION, &generation);
   return generation;
@@ -860,8 +848,9 @@ Configuration::calcSizeAlt(ConfigValues * ownConfig)
     { CFG_DB_RESERVED_TRANS_BUFFER_MEM, &reservedTransactionBufferBytes, true },
   };
 
-  ndb_mgm_configuration_iterator db(*(ndb_mgm_configuration*)ownConfig, 0);
-  
+  ndb_mgm_configuration_iterator db(
+      reinterpret_cast<ndb_mgm_configuration *>(ownConfig), 0);
+
   const int sz = sizeof(tmp)/sizeof(AttribStorage);
   for(int i = 0; i<sz; i++){
     if(ndb_mgm_get_int_parameter(&db, tmp[i].paramId, tmp[i].storage)){
@@ -910,20 +899,20 @@ Configuration::calcSizeAlt(ConfigValues * ownConfig)
   }
   
   // tmp
-  ndb_mgm_configuration_iterator * p = m_clusterConfigIter;
+  ndb_mgm_configuration_iterator * iter = m_clusterConfigIter;
 
   Uint32 nodeNo = noOfNodes = 0;
   NodeBitmask nodes;
-  for(ndb_mgm_first(p); ndb_mgm_valid(p); ndb_mgm_next(p), nodeNo++){
+  for(ndb_mgm_first(iter); ndb_mgm_valid(iter); ndb_mgm_next(iter), nodeNo++){
     
     Uint32 nodeId;
     Uint32 nodeType;
     
-    if(ndb_mgm_get_int_parameter(p, CFG_NODE_ID, &nodeId)){
+    if(ndb_mgm_get_int_parameter(iter, CFG_NODE_ID, &nodeId)){
       ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG, msg, "Node data (Id) missing");
     }
     
-    if(ndb_mgm_get_int_parameter(p, CFG_TYPE_OF_SECTION, &nodeType)){
+    if(ndb_mgm_get_int_parameter(iter, CFG_TYPE_OF_SECTION, &nodeType)){
       ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG, msg, "Node data (Type) missing");
     }
     
@@ -1130,15 +1119,14 @@ Configuration::calcSizeAlt(ConfigValues * ownConfig)
     cfg.put(CFG_ACC_OP_RECS, local_operations);
 
 #ifdef VM_TRACE
-    ndbout_c("reservedOperations: %u, reservedLocalScanRecords: %u,"
-             " NODE_RECOVERY_SCAN_OP_RECORDS: %u, "
-             "noOfLocalScanRecords: %u, "
-             "noOfLocalOperations: %u",
-             reservedOperations,
-             reservedLocalScanRecords,
-             NODE_RECOVERY_SCAN_OP_RECORDS,
-             noOfLocalScanRecords,
-             noOfLocalOperations);
+    g_eventLogger->info(
+        "reservedOperations: %u, reservedLocalScanRecords: %u,"
+        " NODE_RECOVERY_SCAN_OP_RECORDS: %u, "
+        "noOfLocalScanRecords: %u, "
+        "noOfLocalOperations: %u",
+        reservedOperations, reservedLocalScanRecords,
+        NODE_RECOVERY_SCAN_OP_RECORDS, noOfLocalScanRecords,
+        noOfLocalOperations);
 #endif
     Uint32 ldm_reserved_operations =
             (reservedOperations / ldmInstances) + EXTRA_LOCAL_OPERATIONS +
@@ -1333,8 +1321,7 @@ Configuration::calcSizeAlt(ConfigValues * ownConfig)
 
   require(cfg.commit(true));
   m_ownConfig = (ndb_mgm_configuration*)cfg.getConfigValues();
-  m_ownConfigIterator = ndb_mgm_create_configuration_iterator
-    (m_ownConfig, 0);
+  m_ownConfigIterator = ndb_mgm_create_configuration_iterator(m_ownConfig, 0);
 }
 
 void
@@ -1348,7 +1335,7 @@ Configuration::setAllRealtimeScheduler()
       if (setRealtimeScheduler(threadInfo[i].pThread,
                                threadInfo[i].type,
                                _realtimeScheduler,
-                               FALSE))
+                               false))
         return;
     }
   }
@@ -1585,7 +1572,7 @@ Configuration::addThread(struct NdbThread* pThread,
   bool real_time;
   if (single_threaded)
   {
-    setRealtimeScheduler(pThread, type, _realtimeScheduler, TRUE);
+    setRealtimeScheduler(pThread, type, _realtimeScheduler, true);
   }
   else if (type == WatchDogThread ||
            type == SocketClientThread ||
@@ -1605,16 +1592,14 @@ Configuration::addThread(struct NdbThread* pThread,
        * breaks.
        */
       real_time = m_thr_config.do_get_realtime_wd();
-      setRealtimeScheduler(pThread, type, real_time, TRUE);
+      setRealtimeScheduler(pThread, type, real_time, true);
     }
     /**
      * main threads are set in ThreadConfig::ipControlLoop
      * as it's handled differently with mt
      */
-    ndbout_c("Started thread, index = %u, id = %d, type = %s",
-             i,
-             NdbThread_GetTid(pThread),
-             type_str);
+    g_eventLogger->info("Started thread, index = %u, id = %d, type = %s", i,
+                        NdbThread_GetTid(pThread), type_str);
     setLockCPU(pThread, type);
   }
   /**
@@ -1649,13 +1634,13 @@ Configuration::yield_main(Uint32 index, bool start)
     if (start)
       setRealtimeScheduler(threadInfo[index].pThread,
                            threadInfo[index].type,
-                           FALSE,
-                           FALSE);
+                           false,
+                           false);
     else
       setRealtimeScheduler(threadInfo[index].pThread,
                            threadInfo[index].type,
-                           TRUE,
-                           FALSE);
+                           true,
+                           false);
   }
 }
 
