@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,17 +22,16 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-
-
-
 /*****************************************************************************
 Name:          Ndb.cpp
 ******************************************************************************/
 
+#include "util/require.h"
 #include <ndb_global.h>
 
 #include "API.hpp"
 #include <md5_hash.hpp>
+#include "my_config.h"
 #include <NdbSleep.h>
 #include <NdbOut.hpp>
 #include <ndb_limits.h>
@@ -166,15 +165,15 @@ Ndb::NDB_connect(Uint32 tNode, Uint32 instance)
   }
 
   NdbTransaction * tConArray = theConnectionArray[tNode];
-  if (instance != 0 && tConArray != 0)
+  if (instance != 0 && tConArray != nullptr)
   {
-    NdbTransaction* prev = 0;
+    NdbTransaction* prev = nullptr;
     NdbTransaction* curr = tConArray;
     while (curr)
     {
       if (refToInstance(curr->m_tcRef) == instance)
       {
-        if (prev != 0)
+        if (prev != nullptr)
         {
           prev->theNext = curr->theNext;
           if (!curr->theNext)
@@ -192,17 +191,17 @@ Ndb::NDB_connect(Uint32 tNode, Uint32 instance)
       curr = curr->theNext;
     }
   }
-  else if (tConArray != NULL)
+  else if (tConArray != nullptr)
   {
     DBUG_RETURN(2);
   }
 
   NdbTransaction * tNdbCon = getNdbCon();	// Get free connection object.
-  if (tNdbCon == NULL) {
+  if (tNdbCon == nullptr) {
     DBUG_RETURN(4);
   }//if
   NdbApiSignal*	tSignal = getSignal();		// Get signal object
-  if (tSignal == NULL) {
+  if (tSignal == nullptr) {
     releaseNdbCon(tNdbCon);
     DBUG_RETURN(4);
   }//if
@@ -267,12 +266,12 @@ Ndb::getConnectedNdbTransaction(Uint32 nodeId, Uint32 instance){
   NdbTransaction* next = theConnectionArray[nodeId];
   if (instance != 0)
   {
-    NdbTransaction * prev = 0;
+    NdbTransaction * prev = nullptr;
     while (next)
     {
       if (refToInstance(next->m_tcRef) == instance)
       {
-        if (prev != 0)
+        if (prev != nullptr)
         {
           assert(false); // Should have been moved in NDB_connect
           prev->theNext = next->theNext;
@@ -290,12 +289,12 @@ Ndb::getConnectedNdbTransaction(Uint32 nodeId, Uint32 instance){
       next = next->theNext;
     }
     assert(false); // !!
-    return 0;
+    return nullptr;
   }
 found_first:
   removeConnectionArray(next, nodeId);
 found_middle:
-  next->theNext = NULL;
+  next->theNext = nullptr;
 
   return next;
 }//Ndb::getConnectedNdbTransaction()
@@ -321,7 +320,7 @@ Ndb::doDisconnect()
    * theTransactionList.
    */
   tNdbCon = theTransactionList;
-  while (tNdbCon != NULL) {
+  while (tNdbCon != nullptr) {
     tNdbCon->releaseOperations();
     tNdbCon->releaseLockHandles();
     tNdbCon = tNdbCon->theNext;
@@ -332,7 +331,7 @@ Ndb::doDisconnect()
    * the now cleaned up, theTransactionList.
    */
   tNdbCon = theTransactionList;
-  while (tNdbCon != NULL) {
+  while (tNdbCon != nullptr) {
     NdbTransaction* tmpNdbCon = tNdbCon;
     tNdbCon = tNdbCon->theNext;
     releaseConnectToNdb(tmpNdbCon);
@@ -349,7 +348,7 @@ Ndb::doDisconnect()
   for (i = 0; i < tNoOfDbNodes; i++) {
     Uint32 tNode = theDBnodes[i];
     tNdbCon = theConnectionArray[tNode];
-    while (tNdbCon != NULL) {
+    while (tNdbCon != nullptr) {
       NdbTransaction* tmpNdbCon = tNdbCon;
       tNdbCon = tNdbCon->theNext;
       releaseConnectToNdb(tmpNdbCon);
@@ -414,7 +413,7 @@ NdbTransaction* computeHash()
 
 Return Value:   Returns 0 for success, NDBAPI error code otherwise
 Remark:         Computes the distribution hash value for a row with the
-                supplied distribtion key values.
+                supplied distribution key values.
                 Only relevant for natively partitioned tables.
 *****************************************************************************/ 
 int
@@ -429,7 +428,7 @@ Ndb::computeHash(Uint32 *retval,
   const NdbColumnImpl* const * cols = impl->m_columns.getBase();
   Uint32 len;
   unsigned char *pos, *bufEnd;
-  void* malloced_buf = NULL;
+  void* malloced_buf = nullptr;
 
   Uint32 colcnt = impl->m_columns.size();
   Uint32 parts = impl->m_noOfDistributionKeys;
@@ -449,11 +448,11 @@ Ndb::computeHash(Uint32 *retval,
 
   for (Uint32 i = 0; i<parts; i++)
   {
-    if (unlikely(keyData[i].ptr == 0))
+    if (unlikely(keyData[i].ptr == nullptr))
       goto enullptr;
   }
 
-  if (unlikely(keyData[parts].ptr != 0))
+  if (unlikely(keyData[parts].ptr != nullptr))
     goto emissingnullptr;
 
   const NdbColumnImpl* partcols[NDB_MAX_NO_OF_ATTRIBUTES_IN_KEY];
@@ -466,7 +465,7 @@ Ndb::computeHash(Uint32 *retval,
       partcols[j++] = cols[i];
     }
   }
-  DBUG_ASSERT(j == parts);
+  assert(j == parts);
 
   for (Uint32 i = 0; i<parts; i++)
   {
@@ -485,35 +484,29 @@ Ndb::computeHash(Uint32 *retval,
     if (unlikely(lb == 0 && keyData[i].len != maxlen))
       goto emalformedkey;
     
-    if (partcols[i]->m_cs != NULL)
+    if (partcols[i]->m_cs != nullptr)
       len = NdbSqlUtil::strnxfrm_hash_len(partcols[i]->m_cs, (maxlen - lb));
 
     len = (lb + len + 3) & ~(Uint32)3;
     sumlen += len;
   }
 
-  if (!buf)
+  while (true)
   {
-    bufLen = sumlen;
-    bufLen += sizeof(Uint64); /* add space for potential alignment */
-    buf = malloc(bufLen);
-    if (unlikely(buf == 0))
-      return 4000;
-    malloced_buf = buf; /* Remember to free */
-    assert(bufLen > sumlen);
-  }
-
-  {
-    /* Get 64-bit aligned ptr required for hashing */
+    if (buf == nullptr)
+    {
+      bufLen = sumlen;
+      buf = malloc(bufLen);
+      if (unlikely(buf == nullptr))
+        return 4000;
+      malloced_buf = buf; /* Remember to free */
+    }
     assert(bufLen != 0);
-    UintPtr org = UintPtr(buf);
-    UintPtr use = (org + 7) & ~(UintPtr)7;
 
-    buf = (void*)use;
-    bufLen -= Uint32(use - org);
-
-    if (unlikely(sumlen > bufLen))
-      goto ebuftosmall;
+    if (likely(sumlen <= bufLen))
+      break;
+    require(malloced_buf == nullptr);
+    buf = nullptr;
   }
 
   pos= (unsigned char*) buf;
@@ -528,9 +521,13 @@ Ndb::computeHash(Uint32 *retval,
     if ((cs = partcols[i]->m_cs))
     {
       const Uint32 maxlen = (partcols[i]->m_attrSize * partcols[i]->m_arraySize) - lb;
-      int n = NdbSqlUtil::strnxfrm_hash(cs, partcols[i]->m_type,
-                                   pos, bufEnd-pos, 
-                                   ((uchar*)keyData[i].ptr)+lb, len, maxlen);
+      int n = NdbSqlUtil::strnxfrm_hash(cs,
+                                        partcols[i]->m_type,
+                                        pos,
+                                        bufEnd - pos,
+                                        ((const uchar*)keyData[i].ptr) + lb,
+                                        len,
+                                        maxlen);
 
       if (unlikely(n == -1))
 	goto emalformedstring;
@@ -554,9 +551,10 @@ Ndb::computeHash(Uint32 *retval,
   }
   len = Uint32(UintPtr(pos) - UintPtr(buf));
   assert((len & 3) == 0);
+  require(len <= bufLen);
 
   Uint32 values[4];
-  md5_hash(values, (const Uint64*)buf, len >> 2);
+  md5_hash(values, (const char*)buf, len);
   
   if (retval)
   {
@@ -580,9 +578,6 @@ emissingnullptr:
 elentosmall:
   return 4277;
 
-ebuftosmall:
-  return 4278;
-
 emalformedstring:
   if (malloced_buf)
     free(malloced_buf);
@@ -593,6 +588,8 @@ emalformedkey:
   return 4280;
 }
 
+static inline Uint32 pad4(Uint32 len) { return (len + 3)/4*4; }
+
 int
 Ndb::computeHash(Uint32 *retval,
                  const NdbRecord *keyRec,
@@ -601,7 +598,7 @@ Ndb::computeHash(Uint32 *retval,
 {
   Uint32 len;
   unsigned char *pos, *bufEnd;
-  void* malloced_buf = NULL;
+  void* malloced_buf = nullptr;
 
   Uint32 parts = keyRec->distkey_index_length;
 
@@ -620,22 +617,12 @@ Ndb::computeHash(Uint32 *retval,
      * the Distr key.
      */
     bufLen = keyRec->m_keyLenInWords << 2;
-    bufLen += sizeof(Uint64); /* add space for potential alignment */
     buf = malloc(bufLen);
-    if (unlikely(buf == 0))
+    if (unlikely(buf == nullptr))
       return 4000;
     malloced_buf = buf; /* Remember to free */
   }
-
-  {
-    /* Get 64-bit aligned address as required for hashing */
-    assert(bufLen != 0);
-    UintPtr org = UintPtr(buf);
-    UintPtr use = (org + 7) & ~(UintPtr)7;
-
-    buf = (void*)use;
-    bufLen -= Uint32(use - org);
-  }
+  assert(bufLen != 0);
 
   pos= (unsigned char*) buf;
   bufEnd = pos + bufLen;
@@ -647,7 +634,7 @@ Ndb::computeHash(Uint32 *retval,
 
     Uint32 len;
     Uint32 maxlen = keyAttr.maxSize;
-    unsigned char *src= (unsigned char*)keyData + keyAttr.offset;
+    const unsigned char* src = (const unsigned char*)keyData + keyAttr.offset;
 
     if (keyAttr.flags & NdbRecord::IsVar1ByteLen)
     {
@@ -689,13 +676,21 @@ Ndb::computeHash(Uint32 *retval,
                                          pos, bufEnd-pos,
                                          src, len, maxlen);
       if (unlikely(n == -1))
+      {
+        if (NdbSqlUtil::strnxfrm_hash_len(cs, maxlen) > bufEnd - pos)
+          goto ebuftosmall;
         goto emalformedstring;
+      }
       len = n;
+      if (unlikely(pad4(len) > bufEnd - pos))
+        goto ebuftosmall;
     }
     else
     {
       if (keyAttr.flags & NdbRecord::IsVar1ByteLen)
       {
+        if (unlikely(pad4(len + 1) > bufEnd - pos))
+          goto ebuftosmall;
         *pos= (unsigned char)len;
         memcpy(pos+1, src, len);
         len += 1;
@@ -703,10 +698,16 @@ Ndb::computeHash(Uint32 *retval,
       else if (keyAttr.flags & NdbRecord::IsVar2ByteLen)
       {
         len += 2;
+        if (unlikely(pad4(len) > bufEnd - pos))
+          goto ebuftosmall;
         memcpy(pos, src-2, len);
       }
       else
+      {
+        if (unlikely(pad4(len) > bufEnd - pos))
+          goto ebuftosmall;
         memcpy(pos, src, len);
+      }
     }
     while (len & 3)
     {
@@ -716,9 +717,10 @@ Ndb::computeHash(Uint32 *retval,
   }
   len = Uint32(UintPtr(pos) - UintPtr(buf));
   assert((len & 3) == 0);
+  require(len <= bufLen);
 
   Uint32 values[4];
-  md5_hash(values, (const Uint64*)buf, len >> 2);
+  md5_hash(values, (const char*)buf, len);
   
   if (retval)
   {
@@ -733,7 +735,9 @@ Ndb::computeHash(Uint32 *retval,
 euserdeftable:
   return 4544;
 
-  
+ebuftosmall:
+  return 4278;
+
 emalformedstring:
   if (malloced_buf)
     free(malloced_buf);
@@ -752,7 +756,7 @@ Ndb::startTransaction(const NdbRecord *keyRec, const char *keyData,
     return startTransaction(keyRec->table, keyRec->table->getPartitionId(hash));
   }
   theError.code = ret;
-  return 0;
+  return nullptr;
 }
 
 NdbTransaction* 
@@ -768,7 +772,7 @@ Ndb::startTransaction(const NdbDictionary::Table *table,
   }
 
   theError.code = ret;
-  return 0;
+  return nullptr;
 }
 
 Uint32
@@ -776,7 +780,7 @@ NdbImpl::select_node(NdbTableImpl *table_impl,
                      const Uint16 *nodes,
                      Uint32 cnt)
 {
-  if (table_impl == NULL)
+  if (table_impl == nullptr)
   {
     return m_ndb_cluster_connection.select_any(this);
   }
@@ -796,7 +800,7 @@ NdbImpl::select_node(NdbTableImpl *table_impl,
      * The reason is that the transaction could be large and involve
      * many more operations not necessarily using the same partition
      * key. The jump to the primary is to a different location domain,
-     * so we keeping the TC local to this domain always seems preferrable
+     * so we keeping the TC local to this domain always seems preferable
      * to picking the perfect path for this operation.
      */
     if (m_optimized_node_selection)
@@ -910,35 +914,17 @@ Ndb::startTransaction(const NdbDictionary::Table *table,
     Uint32 nodeId = 0;
     
     /**
-     * Make this unlikely...assume new interface(s) are prefered
+     * Make this unlikely...assume new interface(s) are preferred
      */
-    if(unlikely(table != 0 && keyData != 0))
+    if(unlikely(table != nullptr && keyData != nullptr))
     {
       NdbTableImpl* impl = &NdbTableImpl::getImpl(*table);
       Uint32 hashValue;
       {
-	Uint32 buf[4];
-        const Uint32 MaxKeySizeInLongWords= (NDB_MAX_KEY_SIZE + 7) / 8;
-        Uint64 tmp[ MaxKeySizeInLongWords ];
-
-        if (keyLen >= sizeof(tmp))
-        {
-          theError.code = 4207;
-          DBUG_RETURN(NULL);
-        }
-	if((UintPtr(keyData) & 7) == 0 && (keyLen & 3) == 0)
-	{
-	  md5_hash(buf, (const Uint64*)keyData, keyLen >> 2);
-	}
-	else
-	{
-          tmp[keyLen/8] = 0;    // Zero out any 64-bit padding
-	  memcpy(tmp, keyData, keyLen);
-	  md5_hash(buf, tmp, (keyLen+3) >> 2);	  
-	}
-	hashValue= buf[1];
+        Uint32 values[4];
+        md5_hash(values, keyData, keyLen);
+        hashValue= values[1];
       }
-      
       const Uint16 *nodes;
       Uint32 cnt= impl->get_nodes(table->getPartitionId(hashValue),  &nodes);
       nodeId = theImpl->select_node(impl, nodes, cnt);
@@ -946,12 +932,12 @@ Ndb::startTransaction(const NdbDictionary::Table *table,
     else
     {
       /* No hint available, calling select_node with zero count */
-      NdbTableImpl* impl = NULL;
-      if (table != NULL)
+      NdbTableImpl* impl = nullptr;
+      if (table != nullptr)
       {
         impl = &NdbTableImpl::getImpl(*table);
       }
-      nodeId = theImpl->select_node(impl, NULL, 0);
+      nodeId = theImpl->select_node(impl, nullptr, 0);
     }
 
     /* TODO : Should call method above rather than duplicate call to
@@ -987,7 +973,7 @@ Ndb::hupp(NdbTransaction* pBuddyTrans)
   DBUG_PRINT("enter", ("trans: %p", pBuddyTrans));
 
   Uint32 aPriority = 0;
-  if (pBuddyTrans == NULL){
+  if (pBuddyTrans == nullptr){
     DBUG_RETURN(startTransaction());
   }
 
@@ -999,7 +985,7 @@ Ndb::hupp(NdbTransaction* pBuddyTrans)
     NdbTransaction* pCon =
       startTransactionLocal(aPriority, nodeId,
                             refToInstance(pBuddyTrans->m_tcRef));
-    if(pCon == NULL)
+    if(pCon == nullptr)
       DBUG_RETURN(NULL);
 
     if (pCon->getConnectedNodeId() != nodeId){
@@ -1055,7 +1041,7 @@ Ndb::startTransactionLocal(Uint32 aPriority, Uint32 nodeId, Uint32 instance)
   NdbTransaction* tConnection;
   Uint64 tFirstTransId = theFirstTransId;
   tConnection = doConnect(nodeId, instance);
-  if (tConnection == NULL) {
+  if (tConnection == nullptr) {
     DBUG_RETURN(NULL);
   }//if
 
@@ -1100,7 +1086,7 @@ Ndb::appendConnectionArray(NdbTransaction *aCon, Uint32 nodeId)
   {
     theConnectionArray[nodeId] = aCon;
   }
-  aCon->theNext = NULL;
+  aCon->theNext = nullptr;
   theConnectionArrayLast[nodeId] = aCon;
 }
 
@@ -1122,7 +1108,7 @@ Ndb::removeConnectionArray(NdbTransaction *first, Uint32 nodeId)
   NdbTransaction *next = first->theNext;
   if (!next)
   {
-    theConnectionArray[nodeId] = theConnectionArrayLast[nodeId] = NULL;
+    theConnectionArray[nodeId] = theConnectionArrayLast[nodeId] = nullptr;
   }
   else
   {
@@ -1143,7 +1129,7 @@ Ndb::closeTransaction(NdbTransaction* aConnection)
   NdbTransaction* tCon;
   NdbTransaction* tPreviousCon = nullptr;
 
-  if (aConnection == NULL) {
+  if (aConnection == nullptr) {
 //-----------------------------------------------------
 // closeTransaction called on NULL pointer, destructive
 // application behaviour.
@@ -1174,7 +1160,7 @@ Ndb::closeTransaction(NdbTransaction* aConnection)
     theTransactionList = tCon->next();	// from the transaction list.
   } else { 
     while (aConnection != tCon) {
-      if (tCon == NULL) {
+      if (tCon == nullptr) {
 //-----------------------------------------------------
 // closeTransaction called on non-existing transaction
 //-----------------------------------------------------
@@ -1285,7 +1271,7 @@ Parameters:     aTableName (IN) : The table name.
                 step       (IN) : Specifies the step between the 
                                   autoincrement values.
                 start      (IN) : Start value for first value
-Returns:        0 if succesful, -1 if error encountered
+Returns:        0 if successful, -1 if error encountered
 Remark:		Returns a new autoincrement value to the application.
                 The autoincrement values can be increased by steps
                 (default 1) and a number of values can be prefetched
@@ -1298,11 +1284,20 @@ Ndb::getAutoIncrementValue(const char* aTableName,
 {
   DBUG_ENTER("Ndb::getAutoIncrementValue");
   ASSERT_NOT_MYSQLD;
+
+  // Check that db and schema name is set
+  if (theImpl->m_dbname.empty() ||
+      theImpl->m_schemaname.empty())
+  {
+    theError.code = 4377;
+    DBUG_RETURN(-1);
+  }
+
   BaseString internal_tabname(internalize_table_name(aTableName));
 
   Ndb_local_table_info *info=
     theDictionary->get_local_table_info(internal_tabname);
-  if (info == 0) {
+  if (info == nullptr) {
     theError.code = theDictionary->getNdbError().code;
     DBUG_RETURN(-1);
   }
@@ -1321,13 +1316,13 @@ Ndb::getAutoIncrementValue(const NdbDictionary::Table * aTable,
 {
   DBUG_ENTER("Ndb::getAutoIncrementValue");
   ASSERT_NOT_MYSQLD;
-  assert(aTable != 0);
+  assert(aTable != nullptr);
   const NdbTableImpl* table = & NdbTableImpl::getImpl(*aTable);
   const BaseString& internal_tabname = table->m_internalName;
 
   Ndb_local_table_info *info=
     theDictionary->get_local_table_info(internal_tabname);
-  if (info == 0) {
+  if (info == nullptr) {
     theError.code = theDictionary->getNdbError().code;
     DBUG_RETURN(-1);
   }
@@ -1344,7 +1339,7 @@ Ndb::getAutoIncrementValue(const NdbDictionary::Table * aTable,
                            Uint32 cacheSize, Uint64 step, Uint64 start)
 {
   DBUG_ENTER("Ndb::getAutoIncrementValue");
-  assert(aTable != 0);
+  assert(aTable != nullptr);
   const NdbTableImpl* table = & NdbTableImpl::getImpl(*aTable);
 
   if (getTupleIdFromNdb(table, range, autoValue, cacheSize, step, start) == -1)
@@ -1447,7 +1442,7 @@ int readAutoIncrementValue( const char* aTableName,
 
 Parameters:     aTableName (IN) : The table name.
                 autoValue  (OUT) : The current autoincrement value
-Returns:        0 if succesful, -1 if error encountered
+Returns:        0 if successful, -1 if error encountered
 Remark:         Returns the current autoincrement value to the application.
 ****************************************************************************/
 int
@@ -1456,11 +1451,20 @@ Ndb::readAutoIncrementValue(const char* aTableName,
 {
   DBUG_ENTER("Ndb::readAutoIncrementValue");
   ASSERT_NOT_MYSQLD;
+
+  // Check that db and schema name is set
+  if (theImpl->m_dbname.empty() ||
+      theImpl->m_schemaname.empty())
+  {
+    theError.code = 4377;
+    DBUG_RETURN(-1);
+  }
+
   BaseString internal_tabname(internalize_table_name(aTableName));
 
   Ndb_local_table_info *info=
     theDictionary->get_local_table_info(internal_tabname);
-  if (info == 0) {
+  if (info == nullptr) {
     theError.code = theDictionary->getNdbError().code;
     DBUG_RETURN(-1);
   }
@@ -1478,13 +1482,13 @@ Ndb::readAutoIncrementValue(const NdbDictionary::Table * aTable,
 {
   DBUG_ENTER("Ndb::readAutoIncrementValue");
   ASSERT_NOT_MYSQLD;
-  assert(aTable != 0);
+  assert(aTable != nullptr);
   const NdbTableImpl* table = & NdbTableImpl::getImpl(*aTable);
   const BaseString& internal_tabname = table->m_internalName;
 
   Ndb_local_table_info *info=
     theDictionary->get_local_table_info(internal_tabname);
-  if (info == 0) {
+  if (info == nullptr) {
     theError.code = theDictionary->getNdbError().code;
     DBUG_RETURN(-1);
   }
@@ -1500,7 +1504,7 @@ Ndb::readAutoIncrementValue(const NdbDictionary::Table * aTable,
                             TupleIdRange & range, Uint64 & autoValue)
 {
   DBUG_ENTER("Ndb::readAutoIncrementValue");
-  assert(aTable != 0);
+  assert(aTable != nullptr);
   const NdbTableImpl* table = & NdbTableImpl::getImpl(*aTable);
 
   if (readTupleIdFromNdb(table, range, autoValue) == -1)
@@ -1541,7 +1545,7 @@ int setAutoIncrementValue( const char* aTableName,
 Parameters:     aTableName (IN) : The table name.
                 autoValue  (IN) : The new autoincrement value
                 modify     (IN) : Modify existing value (not initialization)
-Returns:        0 if succesful, -1 if error encountered
+Returns:        0 if successful, -1 if error encountered
 Remark:         Sets a new autoincrement value for the application.
 ****************************************************************************/
 int
@@ -1550,11 +1554,20 @@ Ndb::setAutoIncrementValue(const char* aTableName,
 {
   DBUG_ENTER("Ndb::setAutoIncrementValue");
   ASSERT_NOT_MYSQLD;
+
+  // Check that db and schema name is set
+  if (theImpl->m_dbname.empty() ||
+      theImpl->m_schemaname.empty())
+  {
+    theError.code = 4377;
+    DBUG_RETURN(-1);
+  }
+
   BaseString internal_tabname(internalize_table_name(aTableName));
 
   Ndb_local_table_info *info=
     theDictionary->get_local_table_info(internal_tabname);
-  if (info == 0) {
+  if (info == nullptr) {
     theError.code = theDictionary->getNdbError().code;
     DBUG_RETURN(-1);
   }
@@ -1571,13 +1584,13 @@ Ndb::setAutoIncrementValue(const NdbDictionary::Table * aTable,
 {
   DBUG_ENTER("Ndb::setAutoIncrementValue");
   ASSERT_NOT_MYSQLD;
-  assert(aTable != 0);
+  assert(aTable != nullptr);
   const NdbTableImpl* table = & NdbTableImpl::getImpl(*aTable);
   const BaseString& internal_tabname = table->m_internalName;
 
   Ndb_local_table_info *info=
     theDictionary->get_local_table_info(internal_tabname);
-  if (info == 0) {
+  if (info == nullptr) {
     theError.code = theDictionary->getNdbError().code;
     DBUG_RETURN(-1);
   }
@@ -1593,7 +1606,7 @@ Ndb::setAutoIncrementValue(const NdbDictionary::Table * aTable,
                            bool modify)
 {
   DBUG_ENTER("Ndb::setAutoIncrementValue");
-  assert(aTable != 0);
+  assert(aTable != nullptr);
   const NdbTableImpl* table = & NdbTableImpl::getImpl(*aTable);
 
   if (setTupleIdInNdb(table, range, autoValue, modify) == -1)
@@ -1671,7 +1684,7 @@ int Ndb::initAutoIncrement()
   setDatabaseName(currentDb.c_str());
   setDatabaseSchemaName(currentSchema.c_str());
 
-  if (m_sys_tab_0 == NULL) {
+  if (m_sys_tab_0 == nullptr) {
     assert(theDictionary->m_error.code != 0);
     theError.code = theDictionary->m_error.code;
     return -1;
@@ -1722,8 +1735,8 @@ Ndb::opTupleIdOnNdb(const NdbTableImpl* table,
   DBUG_PRINT("enter", ("table: %u  value: %lu  op: %u",
                        aTableId, (ulong) opValue, op));
 
-  NdbTransaction*    tConnection = NULL;
-  NdbOperation*      tOperation = NULL;
+  NdbTransaction*    tConnection = nullptr;
+  NdbOperation*      tOperation = nullptr;
   Uint64             tValue;
   NdbRecAttr*        tRecAttrResult;
 
@@ -1737,11 +1750,11 @@ Ndb::opTupleIdOnNdb(const NdbTableImpl* table,
                                        (const char *) &aTableId,
                                        sizeof(Uint32));
 
-  if (tConnection == NULL)
+  if (tConnection == nullptr)
     goto error_handler;
 
   tOperation = tConnection->getNdbOperation(m_sys_tab_0);
-  if (tOperation == NULL)
+  if (tOperation == nullptr)
     goto error_handler;
 
   switch (op)
@@ -1816,19 +1829,19 @@ Ndb::opTupleIdOnNdb(const NdbTableImpl* table,
 error_handler:
   DBUG_PRINT("error", ("ndb=%d con=%d op=%d",
              theError.code,
-             tConnection != NULL ? tConnection->theError.code : -1,
-             tOperation != NULL ? tOperation->theError.code : -1));
+             tConnection != nullptr ? tConnection->theError.code : -1,
+             tOperation != nullptr ? tOperation->theError.code : -1));
 
-  if (theError.code == 0 && tConnection != NULL)
+  if (theError.code == 0 && tConnection != nullptr)
     theError.code = tConnection->theError.code;
-  if (theError.code == 0 && tOperation != NULL)
+  if (theError.code == 0 && tOperation != nullptr)
     theError.code = tOperation->theError.code;
-  DBUG_ASSERT(theError.code != 0);
+  assert(theError.code != 0);
 
   NdbError savedError;
   savedError = theError;
 
-  if (tConnection != NULL)
+  if (tConnection != nullptr)
     this->closeTransaction(tConnection);
 
   theError = savedError;
@@ -1854,7 +1867,6 @@ convertEndian(Uint32 Data)
 #endif
 }
 
-// <internal>
 Ndb_cluster_connection &
 Ndb::get_ndb_cluster_connection()
 {
@@ -1866,16 +1878,21 @@ const char * Ndb::getCatalogName() const
   return theImpl->m_dbname.c_str();
 }
 
-int Ndb::setCatalogName(const char * a_catalog_name)
+int Ndb::setCatalogName(const char * catalog_name)
 {
-  // TODO can table_name_separator be escaped?
-  if (a_catalog_name && ! strchr(a_catalog_name, table_name_separator)) {
-    if (!theImpl->m_dbname.assign(a_catalog_name) ||
-        theImpl->update_prefix())
-    {
-      theError.code = 4000;
-      return -1;
-    }
+  // Catalog name parameter is required and may not contain the separator (i.e
+  // forward slash) used for the internal name format
+  if (catalog_name == nullptr ||
+     strchr(catalog_name, table_name_separator))
+  {
+    theError.code = 4118; // Invalid parameter
+    return -1;
+  }
+
+  if (!theImpl->m_dbname.assign(catalog_name))
+  {
+    theError.code = 4000;
+    return -1;
   }
   return 0;
 }
@@ -1885,20 +1902,24 @@ const char * Ndb::getSchemaName() const
   return theImpl->m_schemaname.c_str();
 }
 
-int Ndb::setSchemaName(const char * a_schema_name)
+int Ndb::setSchemaName(const char * schema_name)
 {
-  // TODO can table_name_separator be escaped?
-  if (a_schema_name && ! strchr(a_schema_name, table_name_separator)) {
-    if (!theImpl->m_schemaname.assign(a_schema_name) ||
-        theImpl->update_prefix())
-    {
-      theError.code = 4000;
-      return -1;
-    }
+  // Schema name parameter is required and may not contain the separator (i.e
+  // forward slash) used for the internal name format
+  if (schema_name == nullptr ||
+     strchr(schema_name, table_name_separator))
+  {
+    theError.code = 4118; // Invalid parameter
+    return -1;
+  }
+
+  if (!theImpl->m_schemaname.assign(schema_name))
+  {
+    theError.code = 4000;
+    return -1;
   }
   return 0;
 }
-// </internal>
  
 const char* Ndb::getNdbObjectName() const
 {
@@ -1957,18 +1978,9 @@ int Ndb::setDatabaseAndSchemaName(const NdbDictionary::Table* t)
         sprintf(buf, "%.*s", (int) (s2 - (s1 + 1)), s1 + 1);
         setDatabaseSchemaName(buf);
 #ifdef VM_TRACE
-        // verify that m_prefix looks like abc/def/
-        const char* s0 = theImpl->m_prefix.c_str();
-        const char* s1 = s0 ? strchr(s0, table_name_separator) : 0;
-        const char* s2 = s1 ? strchr(s1 + 1, table_name_separator) : 0;
-        if (!(s1 && s1 != s0 && s2 && s2 != s1 + 1 && *(s2 + 1) == 0))
-        {
-          ndbout_c("t->m_impl.m_internalName.c_str(): %s", t->m_impl.m_internalName.c_str());
-          ndbout_c("s0: %s", s0);
-          ndbout_c("s1: %s", s1);
-          ndbout_c("s2: %s", s2);
-          assert(s1 && s1 != s0 && s2 && s2 != s1 + 1 && *(s2 + 1) == 0);
-        }
+        // verify that both db and schema name are set
+        assert(theImpl->m_dbname.length());
+        assert(theImpl->m_schemaname.length());
 #endif
         return 0;
       }
@@ -1979,7 +1991,7 @@ int Ndb::setDatabaseAndSchemaName(const NdbDictionary::Table* t)
  
 bool Ndb::usingFullyQualifiedNames()
 {
-  return fullyQualifiedNames;
+  return true; // Always on
 }
  
 const char *
@@ -2004,12 +2016,6 @@ Ndb::externalizeTableName(const char * internalTableName, bool fullyQualifiedNam
     return internalTableName;
 }
 
-const char *
-Ndb::externalizeTableName(const char * internalTableName)
-{
-  return externalizeTableName(internalTableName, usingFullyQualifiedNames());
-}
-
 const char *
 Ndb::externalizeIndexName(const char * internalIndexName, bool fullyQualifiedNames)
 {
@@ -2036,13 +2042,6 @@ Ndb::externalizeIndexName(const char * internalIndexName, bool fullyQualifiedNam
   }
 }
 
-const char *
-Ndb::externalizeIndexName(const char * internalIndexName)
-{
-  return externalizeIndexName(internalIndexName, usingFullyQualifiedNames());
-}
-
-
 const BaseString
 Ndb::internalize_table_name(const char *external_name) const
 {
@@ -2050,105 +2049,30 @@ Ndb::internalize_table_name(const char *external_name) const
   DBUG_ENTER("internalize_table_name");
   DBUG_PRINT("enter", ("external_name: %s", external_name));
 
-  if (fullyQualifiedNames)
-  {
-    /* Internal table name format <db>/<schema>/<table>
-       <db>/<schema>/ is already available in m_prefix
-       so just concat the two strings
-     */
 #ifdef VM_TRACE
-    // verify that m_prefix looks like abc/def/
-    const char* s0 = theImpl->m_prefix.c_str();
-    const char* s1 = s0 ? strchr(s0, table_name_separator) : 0;
-    const char* s2 = s1 ? strchr(s1 + 1, table_name_separator) : 0;
-    if (!(s1 && s1 != s0 && s2 && s2 != s1 + 1 && *(s2 + 1) == 0))
-    {
-      ndbout_c("s0: %s", s0);
-      ndbout_c("s1: %s", s1);
-      ndbout_c("s2: %s", s2);
-      assert(s1 && s1 != s0 && s2 && s2 != s1 + 1 && *(s2 + 1) == 0);
-    }
+  // verify that both db and schema name are set
+  assert(theImpl->m_dbname.length());
+  assert(theImpl->m_schemaname.length());
 #endif
-    ret.assfmt("%s%s",
-               theImpl->m_prefix.c_str(),
-               external_name);
-  }
-  else
-    ret.assign(external_name);
+
+  // Internal table name format <db>/<schema>/<table>
+  ret.assfmt("%s%c%s%c%s",
+             theImpl->m_dbname.c_str(), table_name_separator,
+             theImpl->m_schemaname.c_str(), table_name_separator,
+             external_name);
 
   DBUG_PRINT("exit", ("internal_name: %s", ret.c_str()));
   DBUG_RETURN(ret);
 }
 
-const BaseString
-Ndb::old_internalize_index_name(const NdbTableImpl * table,
-				const char * external_name) const
-{
-  BaseString ret;
-  DBUG_ENTER("old_internalize_index_name");
-  DBUG_PRINT("enter", ("external_name: %s, table_id: %d",
-                       external_name, table ? table->m_id : ~0));
-  if (!table)
-  {
-    DBUG_PRINT("error", ("!table"));
-    DBUG_RETURN(ret);
-  }
-
-  if (fullyQualifiedNames)
-  {
-    /* Internal index name format <db>/<schema>/<tabid>/<table> */
-    ret.assfmt("%s%d%c%s",
-               theImpl->m_prefix.c_str(),
-               table->m_id,
-               table_name_separator,
-               external_name);
-  }
-  else
-    ret.assign(external_name);
-
-  DBUG_PRINT("exit", ("internal_name: %s", ret.c_str()));
-  DBUG_RETURN(ret);
-}
-
-const BaseString
-Ndb::internalize_index_name(const NdbTableImpl * table,
-                           const char * external_name) const
-{
-  BaseString ret;
-  DBUG_ENTER("internalize_index_name");
-  DBUG_PRINT("enter", ("external_name: %s, table_id: %d",
-                       external_name, table ? table->m_id : ~0));
-  if (!table)
-  {
-    DBUG_PRINT("error", ("!table"));
-    DBUG_RETURN(ret);
-  }
-
-  if (fullyQualifiedNames)
-  {
-    /* Internal index name format sys/def/<tabid>/<table> */
-    ret.assfmt("%s%d%c%s",
-               theImpl->m_systemPrefix.c_str(),
-               table->m_id,
-               table_name_separator,
-               external_name);
-  }
-  else
-    ret.assign(external_name);
-
-  DBUG_PRINT("exit", ("internal_name: %s", ret.c_str()));
-  DBUG_RETURN(ret);
-}
-
-
 const BaseString
 Ndb::getDatabaseFromInternalName(const char * internalName)
 {
   char * databaseName = new char[strlen(internalName) + 1];
-  if (databaseName == NULL)
+  if (databaseName == nullptr)
   {
     errno = ENOMEM;
-    return BaseString(NULL);
+    return BaseString(nullptr);
   }
   strcpy(databaseName, internalName);
   char *ptr = databaseName;
@@ -2166,10 +2090,10 @@ const BaseString
 Ndb::getSchemaFromInternalName(const char * internalName)
 {
   char * schemaName = new char[strlen(internalName)];
-  if (schemaName == NULL)
+  if (schemaName == nullptr)
   {
     errno = ENOMEM;
-    return BaseString(NULL);
+    return BaseString(nullptr);
   }
   const char *ptr1 = internalName;
    
@@ -2186,15 +2110,15 @@ Ndb::getSchemaFromInternalName(const char * internalName)
   return ret;
 }
 
-unsigned Ndb::get_eventbuf_max_alloc()
+Uint64 Ndb::get_eventbuf_max_alloc()
 {
     return theEventBuffer->m_max_alloc;
 }
 
 void
-Ndb::set_eventbuf_max_alloc(unsigned sz)
+Ndb::set_eventbuf_max_alloc(Uint64 sz)
 {
-  if (theEventBuffer != NULL)
+  if (theEventBuffer != nullptr)
   {
     theEventBuffer->m_max_alloc = sz;
   }
@@ -2235,7 +2159,7 @@ NdbEventOperation* Ndb::createEventOperation(const char* eventName)
     NdbEventOperationImpl *op=
       NdbEventBuffer::getEventOperationImpl(tOp);
     op->m_next= theImpl->m_ev_op;
-    op->m_prev= 0;
+    op->m_prev= nullptr;
     theImpl->m_ev_op= op;
     if (op->m_next)
       op->m_next->m_prev= op;
@@ -2264,7 +2188,7 @@ NdbEventOperation *Ndb::getEventOperation(NdbEventOperation* tOp)
     op= theImpl->m_ev_op;
   if (op)
     return op->m_facade;
-  return 0;
+  return nullptr;
 }
 
 int
@@ -2309,11 +2233,11 @@ Ndb::printOverflowErrorAndExit()
                        getReference(), getNdbObjectName());
   g_eventLogger->error("Ndb Event Buffer : Event buffer out of memory.");
   g_eventLogger->error("Ndb Event Buffer : Fatal error.");
-  Uint32 maxalloc = get_eventbuf_max_alloc();
+  Uint64 maxalloc = get_eventbuf_max_alloc();
   if (maxalloc != 0)
   {
     // limited memory is allocated for event buffer, give recommendation
-    g_eventLogger->error("Ndb Event Buffer : Change eventbuf_max_alloc (Current max_alloc is %u).", maxalloc);
+    g_eventLogger->error("Ndb Event Buffer : Change eventbuf_max_alloc (Current max_alloc is %llu).", maxalloc);
   }
   g_eventLogger->error("Ndb Event Buffer : Consider using the new API.");
   exit(-1);
@@ -2371,13 +2295,13 @@ NdbEventOperation *Ndb::nextEvent()
 
   // Remove the event data from the head
   NdbEventOperation *op = theEventBuffer->nextEvent2();
-  if (op == NULL)
-    return NULL;
+  if (op == nullptr)
+    return nullptr;
 
   if (unlikely(op->isErrorEpoch(&errType)))
   {
     if (errType ==  NdbDictionary::Event::TE_INCONSISTENT)
-      return NULL;
+      return nullptr;
 
     if (errType ==  NdbDictionary::Event::TE_OUT_OF_MEMORY)
       printOverflowErrorAndExit();
@@ -2409,7 +2333,7 @@ Ndb::isConsistentGCI(Uint64 gci)
 const NdbEventOperation*
 Ndb::getNextEventOpInEpoch2(Uint32* iter, Uint32* event_types)
 {
-  return getNextEventOpInEpoch3(iter, event_types, NULL);
+  return getNextEventOpInEpoch3(iter, event_types, nullptr);
 }
 
 const NdbEventOperation*
@@ -2418,15 +2342,15 @@ Ndb::getNextEventOpInEpoch3(Uint32* iter, Uint32* event_types,
 {
   NdbEventOperationImpl* op =
     theEventBuffer->getEpochEventOperations(iter, event_types, cumulative_any_value);
-  if (op != NULL)
+  if (op != nullptr)
     return op->m_facade;
-  return NULL;
+  return nullptr;
 }
 
 const NdbEventOperation*
 Ndb::getGCIEventOperations(Uint32* iter, Uint32* event_types)
 {
-  return getNextEventOpInEpoch3(iter, event_types, NULL);
+  return getNextEventOpInEpoch3(iter, event_types, nullptr);
   /*
    * No event operation is added to gci_ops list for exceptional event data.
    * So it is not possible to get them in event_types. No check needed.
@@ -2501,14 +2425,13 @@ Ndb::printState(const char* fmt, ...)
   vsprintf(buf, fmt, ap);
   va_end(ap);
   NdbMutex_Lock(ndb_print_state_mutex);
-  bool dups = false;
   unsigned i;
   ndbout << buf << " ndb=" << hex << (void*)this << endl;
   for (unsigned n = 0; n < MAX_NDB_NODES; n++) {
     NdbTransaction* con = theConnectionArray[n];
-    if (con != 0) {
+    if (con != nullptr) {
       ndbout << "conn " << n << ":" << endl;
-      while (con != 0) {
+      while (con != nullptr) {
         con->printState();
         con = con->theNext;
       }
@@ -2517,21 +2440,18 @@ Ndb::printState(const char* fmt, ...)
   ndbout << "prepared: " << theNoOfPreparedTransactions<< endl;
   if (checkdups(thePreparedTransactionsArray, theNoOfPreparedTransactions)) {
     ndbout << "!! DUPS !!" << endl;
-    dups = true;
   }
   for (i = 0; i < theNoOfPreparedTransactions; i++)
     thePreparedTransactionsArray[i]->printState();
   ndbout << "sent: " << theNoOfSentTransactions<< endl;
   if (checkdups(theSentTransactionsArray, theNoOfSentTransactions)) {
     ndbout << "!! DUPS !!" << endl;
-    dups = true;
   }
   for (i = 0; i < theNoOfSentTransactions; i++)
     theSentTransactionsArray[i]->printState();
   ndbout << "completed: " << theNoOfCompletedTransactions<< endl;
   if (checkdups(theCompletedTransactionsArray, theNoOfCompletedTransactions)) {
     ndbout << "!! DUPS !!" << endl;
-    dups = true;
   }
   for (i = 0; i < theNoOfCompletedTransactions; i++)
     theCompletedTransactionsArray[i]->printState();
@@ -2550,7 +2470,7 @@ Ndb::getNdbErrorDetail(const NdbError& err, char* buff, Uint32 buffLen) const
   if (!buff)
     DBUG_RETURN(NULL);
 
-  if (err.details != NULL)
+  if (err.details != nullptr)
   {
     DBUG_PRINT("info", ("err.code is %u", err.code));
     switch (err.code) {
@@ -2808,5 +2728,5 @@ Ndb::getClientStatName(Uint32 id) const
   if (likely(id < NumClientStatistics))
     return ClientStatNames[id];
 
-  return NULL;
+  return nullptr;
 }
