@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2013, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -39,8 +39,8 @@
 #include "sql/rpl_info.h"
 #include "sql/rpl_mi.h"
 #include "sql/rpl_msr.h" /* Multi source replication */
+#include "sql/rpl_replica.h"
 #include "sql/rpl_rli.h"
-#include "sql/rpl_slave.h"
 #include "sql/sql_parse.h"
 #include "sql/table.h"
 #include "storage/perfschema/pfs_instr.h"
@@ -53,8 +53,7 @@ static void set_channel_name(void *const, const char &, size_t) {}
 
 static void set_group_name(void *const context, const char &value,
                            size_t length) {
-  struct st_row_connect_status *row =
-      static_cast<struct st_row_connect_status *>(context);
+  auto *row = static_cast<struct st_row_connect_status *>(context);
   const size_t max = UUID_LENGTH;
   length = std::min(length, max);
 
@@ -64,8 +63,7 @@ static void set_group_name(void *const context, const char &value,
 
 static void set_source_uuid(void *const context, const char &value,
                             size_t length) {
-  struct st_row_connect_status *row =
-      static_cast<struct st_row_connect_status *>(context);
+  auto *row = static_cast<struct st_row_connect_status *>(context);
   const size_t max = UUID_LENGTH;
   length = std::min(length, max);
 
@@ -74,8 +72,7 @@ static void set_source_uuid(void *const context, const char &value,
 }
 
 static void set_service_state(void *const context, bool value) {
-  struct st_row_connect_status *row =
-      static_cast<struct st_row_connect_status *>(context);
+  auto *row = static_cast<struct st_row_connect_status *>(context);
 
   row->service_state = value ? PS_RPL_CONNECT_SERVICE_STATE_YES
                              : PS_RPL_CONNECT_SERVICE_STATE_NO;
@@ -159,7 +156,7 @@ bool PFS_index_rpl_connection_status_by_thread::match(Master_info *mi) {
     row.thread_id = 0;
 
     if (mi->slave_running == MYSQL_SLAVE_RUN_CONNECT) {
-      PSI_thread *psi MY_ATTRIBUTE((unused)) = thd_get_psi(mi->info_thd);
+      PSI_thread *psi [[maybe_unused]] = thd_get_psi(mi->info_thd);
 #ifdef HAVE_PSI_THREAD_INTERFACE
       if (psi != nullptr) {
         row.thread_id = PSI_THREAD_CALL(get_thread_internal_id)(psi);
@@ -183,9 +180,10 @@ PFS_engine_table *table_replication_connection_status::create(
 table_replication_connection_status::table_replication_connection_status()
     : PFS_engine_table(&m_share, &m_pos), m_pos(0), m_next_pos(0) {}
 
-table_replication_connection_status::~table_replication_connection_status() {}
+table_replication_connection_status::~table_replication_connection_status() =
+    default;
 
-void table_replication_connection_status::reset_position(void) {
+void table_replication_connection_status::reset_position() {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
@@ -195,7 +193,7 @@ ha_rows table_replication_connection_status::get_row_count() {
   return channel_map.get_max_channels();
 }
 
-int table_replication_connection_status::rnd_next(void) {
+int table_replication_connection_status::rnd_next() {
   Master_info *mi = nullptr;
   channel_map.rdlock();
 
@@ -223,7 +221,8 @@ int table_replication_connection_status::rnd_pos(const void *pos) {
 
   channel_map.rdlock();
 
-  if ((mi = channel_map.get_mi_at_pos(m_pos.m_index))) {
+  mi = channel_map.get_mi_at_pos(m_pos.m_index);
+  if (mi) {
     res = make_row(mi);
   }
 
@@ -243,7 +242,7 @@ int table_replication_connection_status::index_init(uint idx, bool) {
       result = PFS_NEW(PFS_index_rpl_connection_status_by_thread);
       break;
     default:
-      DBUG_ASSERT(false);
+      assert(false);
       break;
   }
   m_opened_index = result;
@@ -251,7 +250,7 @@ int table_replication_connection_status::index_init(uint idx, bool) {
   return 0;
 }
 
-int table_replication_connection_status::index_next(void) {
+int table_replication_connection_status::index_next() {
   Master_info *mi = nullptr;
 
   channel_map.rdlock();
@@ -288,8 +287,8 @@ int table_replication_connection_status::make_row(Master_info *mi) {
   m_row.thread_id_is_null = true;
   m_row.service_state = PS_RPL_CONNECT_SERVICE_STATE_NO;
 
-  DBUG_ASSERT(mi != nullptr);
-  DBUG_ASSERT(mi->rli != nullptr);
+  assert(mi != nullptr);
+  assert(mi->rli != nullptr);
 
   mysql_mutex_lock(&mi->data_lock);
 
@@ -330,7 +329,7 @@ int table_replication_connection_status::make_row(Master_info *mi) {
   }
 
   if (mi->slave_running == MYSQL_SLAVE_RUN_CONNECT) {
-    PSI_thread *psi MY_ATTRIBUTE((unused)) = thd_get_psi(mi->info_thd);
+    PSI_thread *psi [[maybe_unused]] = thd_get_psi(mi->info_thd);
 #ifdef HAVE_PSI_THREAD_INTERFACE
     if (psi != nullptr) {
       m_row.thread_id = PSI_THREAD_CALL(get_thread_internal_id)(psi);
@@ -411,27 +410,28 @@ int table_replication_connection_status::read_row_values(TABLE *table,
                                                          bool read_all) {
   Field *f;
 
-  DBUG_ASSERT(table->s->null_bytes == 1);
+  assert(table->s->null_bytes == 1);
   buf[0] = 0;
 
   for (; (f = *fields); fields++) {
     if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
       switch (f->field_index()) {
         case 0: /** channel_name*/
-          set_field_char_utf8(f, m_row.channel_name, m_row.channel_name_length);
+          set_field_char_utf8mb4(f, m_row.channel_name,
+                                 m_row.channel_name_length);
           break;
         case 1: /** group_name */
           if (m_row.group_name_is_null) {
             f->set_null();
           } else {
-            set_field_char_utf8(f, m_row.group_name, UUID_LENGTH);
+            set_field_char_utf8mb4(f, m_row.group_name, UUID_LENGTH);
           }
           break;
         case 2: /** source_uuid */
           if (m_row.source_uuid_is_null) {
             f->set_null();
           } else {
-            set_field_char_utf8(f, m_row.source_uuid, UUID_LENGTH);
+            set_field_char_utf8mb4(f, m_row.source_uuid, UUID_LENGTH);
           }
           break;
         case 3: /** thread_id */
@@ -458,15 +458,15 @@ int table_replication_connection_status::read_row_values(TABLE *table,
           set_field_ulong(f, m_row.last_error_number);
           break;
         case 9: /*last_error_message*/
-          set_field_varchar_utf8(f, m_row.last_error_message,
-                                 m_row.last_error_message_length);
+          set_field_varchar_utf8mb4(f, m_row.last_error_message,
+                                    m_row.last_error_message_length);
           break;
         case 10: /*last_error_timestamp*/
           set_field_timestamp(f, m_row.last_error_timestamp);
           break;
         case 11: /*last_queued_trx*/
-          set_field_char_utf8(f, m_row.last_queued_trx,
-                              m_row.last_queued_trx_length);
+          set_field_char_utf8mb4(f, m_row.last_queued_trx,
+                                 m_row.last_queued_trx_length);
           break;
         case 12: /*last_queued_trx_original_commit_timestamp*/
           set_field_timestamp(f,
@@ -483,7 +483,8 @@ int table_replication_connection_status::read_row_values(TABLE *table,
           set_field_timestamp(f, m_row.last_queued_trx_end_queue_timestamp);
           break;
         case 16: /*queueing_trx*/
-          set_field_char_utf8(f, m_row.queueing_trx, m_row.queueing_trx_length);
+          set_field_char_utf8mb4(f, m_row.queueing_trx,
+                                 m_row.queueing_trx_length);
           break;
         case 17: /*queueing_trx_original_commit_timestamp*/
           set_field_timestamp(f, m_row.queueing_trx_original_commit_timestamp);
@@ -495,7 +496,7 @@ int table_replication_connection_status::read_row_values(TABLE *table,
           set_field_timestamp(f, m_row.queueing_trx_start_queue_timestamp);
           break;
         default:
-          DBUG_ASSERT(false);
+          assert(false);
       }
     }
   }
